{ **************************************************************************** }
{ *                                                                          * }
{ * PARACHUTE (Nintendo, Game & Watch Wide Screen) Simulator                 * }
{ * by Luca "MADrigal" Antignano, October 2007 (revision: January 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit keys;
{ this unit contains the keys settings panel form implementation               }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TKeysForm = class(TForm)
    // labels
    la_main:     TLabel;
    la_1:        TLabel;
    la_2:        TLabel;
    la_3:        TLabel;
    la_4:        TLabel;

    la_controls: TLabel;
    la_5:        TLabel;
    la_6:        TLabel;

    // comboboxes
    cb_1:        TComboBox;
    cb_2:        TComboBox;
    cb_3:        TComboBox;
    cb_4:        TComboBox;
    cb_5:        TComboBox;
    cb_6:        TComboBox;

    // buttons
    btn_accept:  TButton;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate          (Sender: TObject);
    procedure FormActivate        (Sender: TObject);

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure FormKeyDown         (Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure cbChange            (Sender: TObject);
    procedure btn_acceptClick     (Sender: TObject);

{ * PROCEDURES: SET GAME                                                     * }
    function  CheckKeys:          boolean;
    procedure AssignWord          (iKey: integer; var iValue: integer);
  end;

const
  NUMITEMS = 6;

var
{ * main objects * }
  KeysForm: TKeysForm;

{ * array of objects * }
  aLabel: array[1..NUMITEMS] of TLabel;
  aCombo: array[1..NUMITEMS] of TComboBox;

implementation

uses
  unit1;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TKeysForm.FormCreate(Sender: TObject);
{ initializes the combo-boxes                                                  }
  var
    i: integer;
begin
  // objects-arrays association
  aLabel[1] := la_1; aLabel[2] := la_2; aLabel[3] := la_3;
  aLabel[4] := la_4; aLabel[5] := la_5; aLabel[6] := la_6;

  aCombo[1] := cb_1; aCombo[2] := cb_2; aCombo[3] := cb_3;
  aCombo[4] := cb_4; aCombo[5] := cb_5; aCombo[6] := cb_6;

  // duplicates items list in comboboxes (cb_1_penguin_atk is the only one with
  // a full StringList
  for i := 2 to NUMITEMS do
    aCombo[i].Items := aCombo[1].Items;

  // selects correct item in comboboxes
  for i := 1 to NUMITEMS do
    aCombo[i].ItemIndex := aKeys[i].int - 1;

  // forces combo-boxes values check
  cbChange(sender);
end;





procedure TKeysForm.FormActivate(Sender: TObject);
{ sends focus to the first combo box                                           }
begin
  aCombo[1].SetFocus;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TKeysForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  case Key of
    VK_RETURN: if btn_accept.Enabled then                                       // "ENTER" key
      btn_acceptClick(KeysForm);
    VK_ESCAPE: begin                                                            // "ESC"
      Form1.Enabled := true;
      Close;
    end;
  end; // case..of
end;





procedure TKeysForm.cbChange(Sender: TObject);
{ enables/disables the "accept" button if no duplicate entries in the          }
{ combo-boxes                                                                  }
begin
  Btn_Accept.Enabled := not CheckKeys;
end;





procedure TKeysForm.btn_acceptClick(Sender: TObject);
{ writes values of key settings into the system registry                       }
  var
    i: integer;
begin
  // reads values from system registry into the inputs array, then writes them
  // into the system registry (in case no pre-existing values stored). Lastly
  // assigns key values to the key-inputs array
  for i := 1 to NUMITEMS do
    begin
      aKeys[i].int := aCombo[i].ItemIndex + 1;
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  with Form1 do
    begin
      btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
      btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
      btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
      btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
      btn_alarm_top.Hint  := HINT_ALARM;                                                     // "Alarm"

      btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
      btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

      Enabled := true;
    end;

  Close;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SET GAME                                                     * }
{ *                                                                          * }
{ **************************************************************************** }

function TKeysForm.CheckKeys: boolean;
{ returns TRUE if all keys are different; returns FALSE if any keys settings   }
{ are duplicated; changes the colour of labels accordingly                     }
  var
    i, j: integer;
begin
  CheckKeys := false;

  for i := 1 to NUMITEMS do
    aLabel[i].Font.Color := clWhite;

  for i := 1 to NUMITEMS do
    for j := 1 to NUMITEMS do
      if i <> j then
        if aCombo[i].Text = aCombo[j].Text then
          begin
            aLabel[i].Font.Color := clRed;
            aLabel[j].Font.Color := clRed;
            CheckKeys := true;
          end;
end;





procedure TKeysForm.AssignWord(iKey: integer; var iValue: integer);
{ assigns key values to the key-inputs array                                   }
begin
  case iValue of
    1: aKeys[iKey].word := $30; // 0
    2: aKeys[iKey].word := $31; // 1
    3: aKeys[iKey].word := $32; // 2
    4: aKeys[iKey].word := $33; // 3
    5: aKeys[iKey].word := $34; // 4
    6: aKeys[iKey].word := $35; // 5
    7: aKeys[iKey].word := $36; // 6
    8: aKeys[iKey].word := $37; // 7
    9: aKeys[iKey].word := $38; // 8
   10: aKeys[iKey].word := $39; // 9
   11: aKeys[iKey].word := $41; // A
   12: aKeys[iKey].word := $42; // B
   13: aKeys[iKey].word := $43; // C
   14: aKeys[iKey].word := $44; // D
   15: aKeys[iKey].word := $45; // E
   16: aKeys[iKey].word := $46; // F
   17: aKeys[iKey].word := $47; // G
   18: aKeys[iKey].word := $48; // H
   19: aKeys[iKey].word := $49; // I
   20: aKeys[iKey].word := $4A; // J
   21: aKeys[iKey].word := $4B; // K
   22: aKeys[iKey].word := $4C; // L
   23: aKeys[iKey].word := $4D; // M
   24: aKeys[iKey].word := $4E; // N
   25: aKeys[iKey].word := $4F; // O
   26: aKeys[iKey].word := $50; // P
   27: aKeys[iKey].word := $51; // Q
   28: aKeys[iKey].word := $52; // R
   29: aKeys[iKey].word := $53; // S
   30: aKeys[iKey].word := $54; // T
   31: aKeys[iKey].word := $55; // U
   32: aKeys[iKey].word := $56; // V
   33: aKeys[iKey].word := $57; // W
   34: aKeys[iKey].word := $58; // X
   35: aKeys[iKey].word := $59; // Y
   36: aKeys[iKey].word := $5A; // Z
   37: aKeys[iKey].word := VK_UP;      // UP ARROW KEY
   38: aKeys[iKey].word := VK_DOWN;    // DOWN ARROW KEY
   39: aKeys[iKey].word := VK_LEFT;    // LEFT ARROW KEY
   40: aKeys[iKey].word := VK_RIGHT;   // RIGHT ARROW KEY
   41: aKeys[iKey].word := VK_MENU;    // ALT
   42: aKeys[iKey].word := VK_CONTROL; // CONTROL
   43: aKeys[iKey].word := VK_SHIFT;   // SHIFT
   44: aKeys[iKey].word := VK_SPACE;   // SPACEBAR
  end; // case..of
end;





end.
