{ **************************************************************************** }
{ *                                                                          * }
{ * PARACHUTE (Nintendo, Game & Watch Wide Screen) Simulator                 * }
{ * by Luca "MADrigal" Antignano, October 2007 (revision: January 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;   {1, 2}

    btn_game_a_down:          TImage;       btn_game_a_top:           TImage;   {3, 4}
    btn_game_b_down:          TImage;       btn_game_b_top:           TImage;   {5, 6}
    btn_time_down:            TImage;       btn_time_top:             TImage;   {7, 8}
    btn_acl_down:             TImage;       btn_acl_top:              TImage;   {9, 10}
                                            btn_alarm_top:            TImage;   {11}

    btn_left_down:            TImage;       btn_left_top:             TImage;   {12, 13}
    btn_right_down:           TImage;       btn_right_top:            TImage;   {14, 15}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage; {16-18}

    im_panel_alarm_0: TImage; im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage; {19-22}

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage; {23-26}

    im_panel_miss_0: TImage; im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;  {27-30}

    im_panel_game_a: TImage; im_panel_game_b: TImage;                           {31-32}

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage; im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {33-42}

    // on-screen items
    im_heli_0: TImage; im_heli_1: TImage; im_heli_2: TImage;                    {45-45}

    im_para_x1y1: TImage;                                                       {46}
    im_para_x1y2: TImage; im_para_x2y2: TImage;                                 {47-48}
    im_para_x1y3: TImage; im_para_x2y3: TImage; im_para_x3y3: TImage;           {49-51}
    im_para_x1y4: TImage; im_para_x2y4: TImage; im_para_x3y4: TImage;           {52-54}
    im_para_x1y5: TImage; im_para_x2y5: TImage; im_para_x3y5: TImage; im_para_x3y5a: TImage; im_para_x3y5b: TImage; {55-59}
    im_para_x1y6: TImage; im_para_x2y6: TImage; im_para_x3y6: TImage;           {60-62}
    im_para_x1y7: TImage; im_para_x2y7: TImage; im_para_x3y7: TImage;           {63-65}

    im_boat_x1:   TImage; im_boat_x2:   TImage; im_boat_x3:   TImage;           {66-68}

    im_drown_1: TImage; im_drown_2: TImage; im_drown_3: TImage; im_drown_4: TImage; im_drown_5: TImage; im_drown_6: TImage; {69-74}

                        im_shark_2: TImage; im_shark_3: TImage; im_shark_4: TImage; im_shark_5: TImage; im_shark_6: TImage; {75-79}

{ * TIMERS                                                                   * }
    Timer_Chance_Time: TTimer;
    Timer_Clock:       TTimer;
    Timer_Game:        TTimer;
    Timer_Game_Over:   TTimer;
    Timer_Miss:        TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_Heli                 (var p: integer);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Chance_Time;
    procedure Gam_Move_Drowning_Para;
    procedure Gam_Move_Shark;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Chance_TimeTimer        (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
  APP_NAME    = 'Parachute (Nintendo, Game & Watch Wide Screen) Simulator';     // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version 5.02 * More accuracy *';                               // current version (appears on Splash screen)
{}APP_DATE    = 'Released by Luca "MADrigal" Antignano on 14th January, 2021';  // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:lucantignano@gmail.com';                                // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://www.madrigaldesign.it/sim/';                            // link to the website

  REG_KEY     = 'Software\MADrigal\Parachut50';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';

  GAME_SPEED_NORMAL       = 200;                                                // milliseconds between two parachuters ticks
  GAME_SPEED_DELAY        = 120;                                                // milliseconds increase to game ticks if a parachuter is rescued

{}MENU_ITEMS = 13;
  NUM_KEYS = 6;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_chance,
  pfs_got,
  pfs_miss1,
  pfs_miss2,
  pfs_miss3,
  pfs_tick:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_alarm,                                                              // alarm panel
  aIm_panel_miss:          array[0..3]       of TImage;                         // miss marks panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_number:              array[0..9]       of TImage;                         // digits for score/time

  aIm_heli:                array[0..2]       of TImage;                         // helicopter sprites
  aIm_para:                array[1..4, 1..7] of TImage;                         // X=1-3 (x=4 for hanging parachuter on palm tree); Y=1-7
  aIm_boat:                array[1..3]       of TImage;                         // boat sprites
  aIm_drown:               array[1..6]       of TImage;                         // drowning parachuters sprites
  aIm_shark:               array[2..6]       of TImage;                         // sharks sprites

{ * arrays of mixed types * }
  aPara:                   array[1..4, 1..7] of integer;                        // content: 0=empty; 1=flying parachuter; 2=just rescued parachuter
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0/1=ACL 1/2, 2=alarm, 3=time, 4/6=hiscore A/B, 5/7=game A/B, 8=game over
  iScore,                                                                       // score=0-999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999

  iBoatPos,                                                                     // X coordinate of boat
  iMiss,                                                                        // number of misses: 0-3
  aMaxPara                                                                      // maximum amount of parachuters on screen (depending on value of iScore)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bKeyPressed                                                                   // true if left/right arrow keys are kept pressed
                         : boolean;

  // Game variables: loops
  iLoopPara,                                                                    // active parachuter column
  iCounterParaPalm,                                                             // counts how long the parachuter is hanging from the palm in Game B
  iLoopSharks,                                                                  // sharks move at every 30 game ticks (10 iLoopPara loops)
  iLoopHeli,                                                                    // used in Vid_Display_Heli procedure
  iLoopMiss,                                                                    // used in Timer_Miss procedure
  iLoopChance                                                                   // used in Timer_Chance_Time procedure
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_alarm[0] := im_panel_alarm_0; aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3;

  aIm_panel_miss[0] := im_panel_miss_0; aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  aIm_number[0] := im_panel_number_0; aIm_number[1] := im_panel_number_1; aIm_number[2] := im_panel_number_2; aIm_number[3] := im_panel_number_3; aIm_number[4] := im_panel_number_4; aIm_number[5] := im_panel_number_5; aIm_number[6] := im_panel_number_6; aIm_number[7] := im_panel_number_7; aIm_number[8] := im_panel_number_8; aIm_number[9] := im_panel_number_9;

  // on-screen items
  aIm_heli[0] := im_heli_0; aIm_heli[1] := im_heli_1; aIm_heli[2] := im_heli_2;

  aIm_para[1, 7] := im_para_x1y7; aIm_para[2, 7] := im_para_x2y7; aIm_para[3, 7] := im_para_x3y7;
  aIm_para[1, 6] := im_para_x1y6; aIm_para[2, 6] := im_para_x2y6; aIm_para[3, 6] := im_para_x3y6;
  aIm_para[1, 5] := im_para_x1y5; aIm_para[2, 5] := im_para_x2y5; aIm_para[3, 5] := im_para_x3y5;
  aIm_para[1, 4] := im_para_x1y4; aIm_para[2, 4] := im_para_x2y4; aIm_para[3, 4] := im_para_x3y4;
  aIm_para[1, 3] := im_para_x1y3; aIm_para[2, 3] := im_para_x2y3; aIm_para[3, 3] := im_para_x3y3; aIm_para[4, 1] := im_para_x3y5a; aIm_para[4, 2] := im_para_x3y5b;
  aIm_para[1, 2] := im_para_x1y2; aIm_para[2, 2] := im_para_x2y2; aIm_para[3, 2] := nil;
  aIm_para[1, 1] := im_para_x1y1; aIm_para[2, 1] := nil;          aIm_para[3, 1] := nil;

  aIm_boat[1] := im_boat_x1;      aIm_boat[2] := im_boat_x2;      aIm_boat[3] := im_boat_x3;

  aIm_drown[3] := im_drown_3;     aIm_drown[2] := im_drown_2;     aIm_drown[1] := im_drown_1;
  aIm_drown[4] := im_drown_4;     aIm_drown[5] := im_drown_5;     aIm_drown[6] := im_drown_6;

  aIm_shark[3] := im_shark_3;     aIm_shark[2] := im_shark_2;
  aIm_shark[4] := im_shark_4;     aIm_shark[5] := im_shark_5;     aIm_shark[6] := im_shark_6;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"   (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint  := HINT_ALARM;                                                     // "Alarm"
  btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
  btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bKeypressed := false;                                                         // no key is currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_chance); FSOUND_Sample_Free(pfs_got);
  FSOUND_Sample_Free(pfs_miss1);  FSOUND_Sample_Free(pfs_miss2);
  FSOUND_Sample_Free(pfs_miss3);  FSOUND_Sample_Free(pfs_tick);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  for i := 0 to 3 do
    aIm_panel_alarm[i].Visible := b;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);
  i := Ord(b) * 4; Vid_Show_Heli(i);

  for i := 1 to 7 do aIm_para[1, i].Visible := b;
  for i := 2 to 7 do aIm_para[2, i].Visible := b;
  for i := 3 to 7 do aIm_para[3, i].Visible := b;
  for i := 1 to 2 do aIm_para[4, i].Visible := b;
  for i := 1 to 3 do aIm_boat[i].Visible := b;
  for i := 1 to 6 do aIm_drown[i].Visible := b;
  for i := 2 to 6 do aIm_shark[i].Visible := b;
end;





procedure TForm1.Vid_Show_Heli(var p: integer);
{ displays helicopter in various configurations                                }
  var
    i: integer;
begin
  case p of
    0: for i := 0 to 2 do aIm_heli[i].Visible := false;                                                    // nothing visible
    1: begin aIm_heli[0].Visible := true; aIm_heli[1].Visible := false; aIm_heli[2].Visible := false; end; // helicopter only
    2: begin aIm_heli[0].Visible := true; aIm_heli[1].Visible := false; aIm_heli[2].Visible := true;  end; // helicopter + right wing
    3: begin aIm_heli[0].Visible := true; aIm_heli[1].Visible := true;  aIm_heli[2].Visible := false; end; // helicopter + left wing
    4: for i := 0 to 2 do aIm_heli[i].Visible := true;                                                     // everything visible
  end; // case..of
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  aIm_panel_miss[0].Visible := p > 0;                                           // displays/hides item 0

  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                        // thousands (always hidden)

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_number[t div 100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_number[t div  10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_number[t         mod 10].Picture                                             // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL
  if Key = aKeys[5].word then Inp_left_down   else                              // left
  if Key = aKeys[6].word then Inp_right_down;                                   // right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_left_up   else                                // left
  if Key = aKeys[6].word then Inp_right_up;                                     // right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 4 then                                                             // hiscore A ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 6 then                                                             // hiscore B ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 2 then                                                             // alarm ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  iMode := 0; Gam_Set_Mode;                                                     // any mode -> ACL/1
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  iMode := 1; Gam_Set_Mode;                                                     // ACL/1  -> ACL/2
end;





procedure TForm1.Inp_left_down;
{ LEFT button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if (iMode > 3) and (not bKeypressed) then                                     // if (mode is hiscore/game/game over) AND (key not yet pressed)
    begin
      bKeypressed := true;                                                        // key is now pressed (used to prevent from automatic keypresses)

      if iBoatPos > 1 then                                                        // if boat is not yet on the leftmost side
        begin
          aIm_boat[iBoatPos].Visible := false;                                      // hides currently visible boat
          Dec(iBoatPos);                                                            // updates boat coordinate
          aIm_boat[iBoatPos].Visible := true;                                       // displays boat in new position
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ LEFT button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;

  bKeypressed := false;                                                         // key is now released, controller is enabled
end;





procedure TForm1.Inp_right_down;
{ RIGHT button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if (iMode > 3) and (not bKeypressed) then                                     // if (mode is hiscore/game/game over) AND (key not yet pressed)
    begin
      bKeypressed := true;                                                        // key is now pressed (used to prevent from automatic keypresses)

      if iBoatPos < 3 then                                                        // if boat is not yet on the rightmost side
        begin
          aIm_boat[iBoatPos].Visible := false;                                      // hides currently visible boat
          Inc(iBoatPos);                                                            // updates boat coordinates
          aIm_boat[iBoatPos].Visible := true;                                       // shows boat in new position
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ RIGHT button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;

  bKeypressed := false;                                                         // key is now released, controller is enabled
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Chance_Time.Enabled := false;
  Timer_Clock.Enabled       := false;
  Timer_Game.Enabled        := false;
  Timer_Game_Over.Enabled   := false;
  Timer_Miss.Enabled        := false;
end;





procedure TForm1.Gam_Check_Chance_Time;
{ activates the "chance time" bonus at 200 and 500 points                      }
begin
  // chance-time check
  if ( (iScore = 200) or (iScore = 500) )
    and (iMiss > 0) then
    begin
      // bonus animation begins
      Gam_Stop_All_Timers;

      iLoopChance := 1;
      Timer_Chance_Time.Interval := 440;
      Timer_Chance_Time.Enabled := true;
    end;
end;





procedure TForm1.Gam_Move_Drowning_Para;
{ drowning men animation moving, exiting from the right side                   }
  var
    i: integer;
begin
  for i := 6 downto 2 do                                                        // loops drowning para animation
    aIm_drown[i].Visible := aIm_drown[i - 1].Visible;

  aIm_drown[1].Visible := false;                                                // hides drowning para in position 1
end;





procedure TForm1.Gam_Move_Shark;
{ sharks animation, entering, moving, exiting from where he came from          }
  var
    i: integer;
begin
  for i := 6 downto 3 do                                                        // loops sharks animation
    aIm_shark[i].Visible := aIm_shark[i - 1].Visible;

  aIm_shark[2].Visible := iLoopSharks = 10;                                     // new shark appears 10 ticks after the previous one has gone

  Inc(iLoopSharks);

  if iLoopSharks > 15 then                                                      // loops sharks animation, shark is generated at every 15 ticks
    iLoopSharks := 1;
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    i,
    iHundreds : integer;
begin
  i := iScore;
  iHundreds := 0;

  while i > 99 do
    begin
      Dec(i, 100);
      Inc(iHundreds);
    end;

  // number of max parachuters on screen based on score 0..99
  case i of                                                                     
     0..3  : aMaxPara := 1;
     4..7  : aMaxPara := 2;
     8..9  : aMaxPara := 1;
    10..13 : aMaxPara := 2;
    14..17 : aMaxPara := 3;
    18..19 : aMaxPara := 2;
    20..23 : aMaxPara := 3;
    24..27 : aMaxPara := 4;
    28..43 : aMaxPara := 3;
    44..46 : aMaxPara := 4;
    47..55 : aMaxPara := 5;
    56..65 : aMaxPara := 4;
    66..67 : aMaxPara := 5;
    68..74 : aMaxPara := 4;
    75..76 : aMaxPara := 5;
    77..84 : aMaxPara := 4;
    85..87 : aMaxPara := 5;
    88..93 : aMaxPara := 4;
    94..99 : aMaxPara := 5;
  end; // case..of

  // increase in the number of max parachuters based on hundreds in the score
  case iHundreds of                                                             
    1: Inc(aMaxPara, 1);
    2: Inc(aMaxPara, 1);
    3: Inc(aMaxPara, 1);
    4: Inc(aMaxPara, 1);
    5: Inc(aMaxPara, 2);
    6: Inc(aMaxPara, 2);
    7: Inc(aMaxPara, 2);
    8: Inc(aMaxPara, 3);
    9: Inc(aMaxPara, 3);
  end; // case..of

  // maximum 6 parachuters on screen (the ones on the palm don't count)
  if aMaxPara > 6 then
    aMaxPara := 6;

  // calculate game speed
  Timer_Game.Interval := GAME_SPEED_NORMAL - (iHundreds * 5);

  if Timer_Game.Interval < 165 then
    Timer_Game.Interval := 165;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
  var
    i, j : integer;
begin
  case iMode of
    0: begin                                                                    // ACL 1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         Vid_Show_All(false);
       end;
    1: Vid_Show_All(true);                                                      // ACL 2 (ACL button release)
    2: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         i := 4; Vid_Show_Heli(i);

         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;

         aIm_panel_digit_pos[1].Picture := aIm_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_number[0].Picture;

         aIm_panel_alarm[0].Visible := true;
         aIm_panel_alarm[2].Visible := true;
       end;
    3: begin                                                                    // TIME (Time button release)
         Vid_Show_All(false);

         i := 4; Vid_Show_Heli(i);

         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;
       end;
    4, 6: begin                                                                 // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         iBoatPos := 2; aIm_boat[iBoatPos].Visible := true;

         case iMode of
           4: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           6: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of
       end;
    5, 7: begin                                                                 // GAME A/B (Game A/B button release)
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iMiss := 0;

         for i := 1 to 4 do
           for j := 1 to 7 do
             aPara[i, j] := 0;

         iLoopHeli := 4; Vid_Show_Heli(iLoopHeli);
         iLoopSharks := 1;
         aMaxPara := 1;

         iLoopPara := Random(2) + 1;
         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;
       end;
    8: begin                                                                    // GAME OVER
         // do nothing
       end;
    end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Chance_TimeTimer(Sender: TObject);
{ "chance time" intermission                                                   }
  var
    i: integer;
begin
  if iLoopChance < 20 then                                                      // misses blink 10 times, then game continues
    begin
      if Odd(iLoopChance) then                                                    // 1 loop out of 2 (odd)
        begin
          Snd_play(pfs_chance);
          Vid_Show_Panel_Miss(iMiss);                                               // displays number of current misses

          Timer_Chance_Time.Interval := 225;                                        // makes them disappear in 225 milliseconds
        end
      else                                                                        // 1 loop out of 2 (even)
        begin
          i := 0; Vid_Show_Panel_Miss(i);                                           // displays zero misses

          Timer_Chance_Time.Interval := 275;                                        // makes them appear in 275 milliseconds
        end;
    end
  else                                                                          // game continues
    begin
      iMiss := 0; Vid_Show_Panel_Miss(iMiss);                                     // all misses are canceled

      Timer_Game.Enabled := true;                                                 // game proceeds
      Timer_Chance_Time.Enabled := false;                                         // disables this timer
    end;

  Inc(iLoopChance);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_number[hour mod 10].Picture;            // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_number[min  div 10].Picture;            // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_number[min  mod 10].Picture;            // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game timer! this routine includes almost the whole game implementation  }
  var
    i, j,
    iOnscreenPara,
    iSound:        integer;
    bError:        boolean;
begin
  // parachuters rescued in previous loop must now disappear
  for i := 1 to 3 do
    if aPara[i, 7] = 2 then
      aPara[i, 7] := 0;

  // set game difficulty for this tick
  Gam_Set_Difficulty;                                                           // calculates game difficulty (speed and maximum number of parachuters on screen)

  // initializes variables to be used soon...
  bError := false;                                                              // becomes true if any miss is occuring
  iSound := 0;                                                                  // might become 1, 2 or 3, depending on what happens below...

  // checks for misses
  if (aPara[1, 7] = 1) and (iBoatPos <> 1) then begin aPara[1, 7] := 0; aIm_drown[3].Visible := true; aIm_shark[3].Visible := true; iLoopMiss := 3; bError := true; end;
  if (aPara[2, 7] = 1) and (iBoatPos <> 2) then begin aPara[2, 7] := 0; aIm_drown[2].Visible := true; aIm_shark[2].Visible := true; iLoopMiss := 2; bError := true; end;
  if (aPara[3, 7] = 1) and (iBoatPos <> 3) then begin aPara[3, 7] := 0; aIm_drown[1].Visible := true;                               iLoopMiss := 1; bError := true; end;

  // if no miss has occured, move parachuters, sharks, helicopter, etc...
  if not bError then
    begin
      // heli and shark animations
      Vid_Show_Heli(iLoopHeli);

      if iLoopPara = 1 then
        Gam_Move_Shark;

      // checks collisions for parachuters rescue (1st time)
      if aPara[iBoatPos, 7] = 1 then                                            // checks for parachuter to rescue
        begin
          aPara[iBoatPos, 7] := 2;                                                // got him, make it disappear with next loop
          Inc(iScore);                                                            // increases score

          if iScore >= 999                                                        // resets score to 0 if 999
            then iScore := 0;

          Vid_Show_Panel_Score(iScore, true);                                     // display current score
          iSound := 2;                                                            // get ready to play "got" sound

          Timer_Game.Interval := Timer_Game.Interval + GAME_SPEED_DELAY;          // little pause before next parachuters move

          Gam_Check_Chance_Time;                                                  // checks score for "chance time bonus"
        end;

      // GAME B: parachuter on column 3 might end on the palm, etc...
      if (iMode = 7) and (iLoopPara = 3) then
        begin

        if aPara[4, 1] = 1 then
          begin
            aPara[4, 1] := 0;
            Inc(iCounterParaPalm);

            iSound := 1;

            if (Random(3) = 0) and (aPara[3, 4] = 0) and (aPara[3, 5] = 0) and (aPara[3, 6] = 0) and (iCounterParaPalm >= 2) then
              aPara[3, 5] := 1                                                    // becomes immediately [3, 5] := 0 and [3, 6] := 1
            else
              aPara[4, 2] := 1;
          end
        else
          if aPara[4, 2] = 1 then
            begin
              iSound := 1;
              aPara[4, 2] := 0;
              aPara[4, 1] := 1;
            end
          else
            if (aPara[3, 5] = 1) and (aPara[4, 1] = 0) and (aPara[4, 2] = 0) then
              if Random(2) = 0 then
                begin
                  iSound := 1;
                  aPara[3, 5] := 0;
                  aPara[4, 1] := 1;
                  iCounterParaPalm := 0;
                end;
        end;

      // on-screen parachuters movement
      for i := 7 downto (iLoopPara + 1) do
        aPara[iLoopPara, i] := aPara[iLoopPara, i - 1];

      aPara[iLoopPara,iLoopPara] := 0;

      // parachuters generation
      iOnscreenPara := 0;

     for i := 1 to 3 do                                                         // counts how many parachuters are on screen (parachuter on palm doesn't count as it is on [4,1] and [4,2])
       for j := 1 to 7 do
         if aPara[i, j] <> 0 then
         Inc(iOnscreenPara);

      if iOnScreenPara < aMaxPara then                                          // if there's "room" for more parachuters
        if Random(5) < 2 then                                                     // 2 chances out of 5 to generate new parachuter
          aPara[iLoopPara, iLoopPara] := 1;

      if iOnscreenPara = 0 then                                                 // forces parachuter generation in case screen is empty
        aPara[iLoopPara, iLoopPara] := 1;

      if aPara[iLoopPara, iLoopPara + 1] > 0 then                               // avoids generation in case just generated a para on that same column
        aPara[iLoopPara, iLoopPara] := 0;

      if (iMode = 7)                                                            // 2 chances out of 5 to avoid generation if in GameB and there is a parachuter on the palm already
        and (iLoopPara = 3)
        and ((aPara[4, 1] = 1) or (aPara[4, 2] = 1))
        and (Random(5) < 2) then
        aPara[3, 3] := 0;

      // checks collisions for parachuter rescue (2nd time)
      if aPara[iBoatPos, 7] = 1 then                                            // checks for parachuter to rescue
        begin
          aPara[iBoatPos, 7] := 2;                                                // got him, make it disappear with next loop
          Inc(iScore);                                                            // increases score

          if iScore >= 999                                                        // resets score to 0 if 999
            then iScore := 0;

          Vid_Show_Panel_Score(iScore, true);                                     // displays current score
          iSound := 2;                                                            // get ready to play "got" sound

          Timer_Game.Interval := Timer_Game.Interval + GAME_SPEED_DELAY;          // little pause before next parachuters move

          Gam_Check_Chance_Time;                                                  // checks score for "chance time bonus"
        end;

      // scans current column in search for existing parachuter (needed for sound)
      iOnscreenPara := 0;

      for i := 1 to 7 do
        if aPara[iLoopPara, i] > 0 then
          Inc(iOnscreenPara);

      if (iOnscreenPara > 0) and (iSound <> 2) then
        iSound := 1;

      // loops parachuters and heli animation
      Inc(iLoopPara);

      if iLoopPara > 3 then
        iLoopPara := 1;

      Inc(iLoopHeli);

      if iLoopHeli > 4 then
        iLoopHeli := 2;
    end
  else
    begin                                                                       // miss occurs!
      Gam_Stop_All_Timers;
      iSound := 3;

      Timer_Miss.Interval := 620;
      Timer_Miss.Enabled := true;
    end;

  // on-screen parachuters visualization
  for i := 1 to 7 do aIm_para[1, i].Visible := aPara[1, i] > 0;
  for i := 2 to 7 do aIm_para[2, i].Visible := aPara[2, i] > 0;
  for i := 3 to 7 do aIm_para[3, i].Visible := aPara[3, i] > 0;
  for i := 1 to 2 do aIm_para[4, i].Visible := aPara[4, i] > 0;

  // plays sound depending on what's happened above
  case iSound of
    1: Snd_Play(pfs_tick);
    2: Snd_Play(pfs_got);
    3: Snd_Play(pfs_miss1);
  end; // case..of
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game goes into "time" mode, five minutes after game is over                  }
begin
  iMode := 3; Gam_Set_Mode;

  Timer_Game_Over.Enabled := false;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occured. executes "miss" intermission                             }
begin
  Inc(iLoopHeli);

  if iLoopHeli > 4 then
    iLoopHeli := 2;

  Vid_Show_Heli(iLoopHeli);

  case iLoopMiss of
    1: begin
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         aIm_shark[2].Visible := true;                                          // forces shark to appear in position 2 (in case it's not yet there)
         Snd_Play(pfs_miss2);
         Timer_Miss.Interval := 330;                                            // needs to be declared in every case, since we're not sure in which column the miss has occured
       end;
    2: begin
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         Snd_Play(pfs_miss2);
         Timer_Miss.Interval := 330;
       end;
    3: begin
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         Snd_Play(pfs_miss2);
         Timer_Miss.Interval := 330;
       end;
    4: begin
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         Snd_Play(pfs_miss2);
         Timer_Miss.Interval := 330;
       end;
    5: begin
         Snd_Play(pfs_miss3);
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         Timer_Miss.Interval := 570;                                            // longer sound to play, then game continues
       end;
    6: begin
         Gam_Move_Drowning_Para; Gam_Move_Shark;
         Inc(iMiss); Vid_Show_Panel_Miss(iMiss);                                   // number of misses increases

         if iMiss < 3 then                                                      // checks number of misses
           Timer_Game.Enabled := true                                             // game continues if misses below 3
         else
           begin                                                                  // game is over, sets mode to "game over"
             if (iMode = 5) and (iScore > iHiScoreA) then                           // records high score to system registry (game A)
               begin
                 iHiScoreA := iScore;
                 iReg.WriteInteger('', 'HiScore1', iHiScoreA);
               end;

             if (iMode = 7) and (iScore > iHiScoreB) then                           // records high score to system registry (game B)
               begin
                 iHiScoreB := iScore;
                 iReg.WriteInteger('', 'HiScore2', iHiScoreB);
               end;

             Gam_Stop_All_Timers;

             Timer_Game_Over.Interval := 30000;
             Timer_Game_Over.Enabled := true;

             iMode := 8; Gam_Set_Mode;                                              // game is over, go back to "time" mode in 5 minutes
           end;

         Timer_Miss.Enabled := false;                                             // disables this timer
       end;
  end; // case..of

  Inc(iLoopMiss);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
